%%%%%%%%
%%% Database for Forecasting
%%%%%%%%
close all; clear all;

%% Load quarterly data
% Command 'dbdload' loads the data from the 'csv' file (save from Excel as
% .csv in the current directory). All the data are now available in the
% database 'd' 
d = dbload('data.csv');

%% Seasonal adjustment
list = dbobjects(d);

for i = 1:length(list)
    if length(list{i})>1
        if strcmp('_u', list{i}(end-1:end))
            d.(list{i}(1:end-2)) = x12(d.(list{i}), Inf, 'mode', 'm');
            d = rmfield(d, list{i});
        end
    end
end

%% Make log of variables
exceptions = {'rn','x_rn'};

list = dbobjects(d);

for i = 1:length(list)
    if isempty(strmatch(list{i},exceptions,'exact'))
        d.(['l' list{i}]) = 100*log(d.(list{i}));
    end
end

%% Declare here real exchange rate
d.lz = d.ls + d.lx_cpi - d.lcpi;

%% Growth rate qoq, yoy
exceptions = {'rn','x_rn'};

list = dbobjects(d);

for i = 1:length(list)
    if isempty(strmatch(list{i}, exceptions,'exact'))
        if length(list{i})>1
            if strcmp('l', list{i}(1:1))
                d.(['dot_' list{i}(2:end)])  = 4*(d.(list{i}) - d.(list{i}){-1});
                d.(['dot4_' list{i}(2:end)]) = d.(list{i}) - d.(list{i}){-4};
            end
        end
    end
end


%% Real variables
% Domestic real interest rate
d.rr = d.rn - d.dot4_cpi;

% Fooreign real interest rate
d.x_rr = d.x_rn - d.dot4_x_cpi;

%% Trends and Gaps - Hodrick-Prescott filter
list = {'rr','lz','x_rr'};

for i = 1:length(list)
    [d.([list{i} '_eq']), d.([list{i} '_gap'])] = hpf(d.(list{i}));
end

d.dot_z_eq = 4*(d.lz_eq - d.lz_eq{-1});

%% Trend and Gap for Output - Band-pass filter
d.lgdp_gap = bpass(d.lgdp,[6 32],inf);
d.lgdp_eq = hpf((d.lgdp-d.lgdp_gap),inf,'lambda',5);
d.dot_gdp_eq = 4*(d.lgdp_eq - d.lgdp_eq{-1});

%% Foreign Output gap - Band-pass filter
d.lx_gdp_gap = bpass(d.lx_gdp,[6 32],inf);

%% Inflation target (history)
d.target = hpf(d.dot_cpi);

%If you want use exact number for the end value (there is 3 as a proxy) uncomment following two lines 
%prior_cpi = tseries(get(d.dot_cpi,'end'),3);
%d.target = hpf(d.dot_cpi,inf,'level',prior_cpi);

%% Compute Exchange rate target over the history
%Exchange rate target is automatically calculated as to equal observed
%exchange rate at the end of the sample
prior_ls_tar = tseries(get(d.ls,'last'),d.ls(end));
d.ls_tar = hpf(d.ls,inf,'lambda',1600,'level',prior_ls_tar);
d.dot_s_tar = 4*(d.ls_tar - d.ls_tar{-1});

%% Expert change in the database
d.shock_prem = tseries(get(d.ls,'range'),2);
% d.rr_eq(qq(2008,4)) = 1;

%% Sets cumulative output gap zero at the end of the observed sample
d.cum_gap = tseries(get(d.lgdp,'end'),0);

%% Save the database
% Database is saved in file 'history.mat'
savestruct('history.mat',d);
dbsave(d,'history.csv');

%% Graphs
% Makes graphs of stylized facts
Plotrng = qq(1997,1):qq(2009,4);

% Specify country
country = 'The Czech Republic - Stylized Facts';

% Report
x = report('heading',country,'dateformat','YYYY:P','color', true);

x = newpage(x);

x = opengraph(x);

  x = tag(x,'Nominal Variables','fontsize','large');

  subplot(2,2,1); hold on;
  plot(Plotrng,[d.dot_cpi, d.dot4_cpi],'linewidth',2,{'color'},{'b';'r'}); title('Inflation'); 
  legend('q-o-q','y-o-y','location','best');
  grid on; box off;
  
  subplot(2,2,2); hold on;
  plot(Plotrng,[d.rn],'linewidth',2); title('Nominal Interest Rate');
  grid on; box off;
  
  subplot(2,2,3); hold on;
  plot(Plotrng,[d.dot_s, d.dot4_s],'linewidth',2,{'color'},{'b';'r'}); title('Nominal Exchange Rate (q-o-q)');
  legend('q-o-q','y-o-y','location','best');
  grid on; box off;
  
  subplot(2,2,4); hold on;
  plot(Plotrng,[d.dot_x_cpi, d.dot4_x_cpi],'linewidth',2,{'color'},{'b';'r'}); title('Foreign Inflation (q-o-q)');
  legend('q-o-q','y-o-y','location','best');
  grid on; box off;
  
x = getgraph(x);

x = newpage(x);

x = opengraph(x);

  x = tag(x,'Real Variables and Trends','fontsize','large');

  subplot(2,2,1); hold on;
  plot(Plotrng,[d.lgdp, d.lgdp_eq],'linewidth',2,{'color'},{'b';'r'}); title('Output'); 
  legend('Output','Trend','location','best');
  grid on; box off;
  
  subplot(2,2,2); hold on;
  plot(Plotrng,[d.rr, d.rr_eq],'linewidth',2,{'color'},{'b';'r'}); title('Real Interest Rate'); 
  legend('Real Interest Rate','Trend','location','best');
  grid on; box off;
  
  subplot(2,2,3); hold on;
  plot(Plotrng,[d.lz, d.lz_eq],'linewidth',2,{'color'},{'b';'r'}); title('Real Exchange Rate'); 
  legend('Real Exchange','Trend','location','best');
  grid on; box off;
  
  subplot(2,2,4); hold on;
  plot(Plotrng,[d.x_rr, d.x_rr_eq],'linewidth',2,{'color'},{'b';'r'}); title('Foreign Real Interest Rate'); 
  legend('Foreign Real Interest Rate','Trend','location','best');
  grid on; box off;
  
x = getgraph(x);

x = newpage(x);

x = opengraph(x);

  x = tag(x,'Gaps','fontsize','large');

  subplot(2,2,1); hold on;
  plot(Plotrng,d.lgdp_gap,'linewidth',2); title('Output Gap'); 
  grid on; box off;
  
  subplot(2,2,2); hold on;
  plot(Plotrng,d.rr_gap,'linewidth',2); title('Real Interest Rate Gap'); 
  grid on; box off;
  
  subplot(2,2,3); hold on;
  plot(Plotrng,d.lz_gap,'linewidth',2); title('Real Exchange Rate Gap'); 
  grid on; box off;
  
  subplot(2,2,4); hold on;
  plot(Plotrng,d.x_rr_gap,'linewidth',2); title('Foreign Real Interest Rate Gap'); 
  grid on; box off;
  
x = getgraph(x);

%% The default is set to generate a PostScript output, but it can be
%% changed to a .pdf (Acrobat) file by:
compile(x,'Stylized_facts.pdf');
compile(x,'Stylized_facts.ps','display',false);
