%%%%%%%%%%
%% Model Properties
%%%%%%%%%%
clear all; close all;

readmodel;

% Loads the model structure from the file 'model.mat' 
m = loadstruct('model.mat');

% Sets the time frame for the simulation 
startsim = qq(2010,1);
endsim = qq(2013,4); % three-year simulation horizon

%% List of shocks
% Temporary shocks to: inflation, output, exchange rate, interest rate
% Creates a list (vector) of shocks and list of their names. Used shocks' names must
% be the names found in the model code (in file 'model.mod') except of the prefix
% 'shock_'
listshocks = {'shk_dot_cpi','shk_lgdp_gap','shk_ls','shk_rn','shk_prem'};
listtitles = {'Inflationary Shock', 'Aggregate Demand Shock', 'Exchange Rate Shock', 'Interest Rate Shock', 'Risk Premium Shock'};

% For each shock a zero database is created (command 'zerodb') and named as 
% database 'd.{shock_name}'
for i = 1:length(listshocks);
    d.(listshocks{i}) = zerodb(m,startsim-4:endsim);
end

%% Calibration of shocks
% Fills the respective databases with the shocks' values for the starting
% point of the simulation (startsim). For simplicity, all shocks are set to
% 1 percent
d.shk_dot_cpi.shk_dot_cpi(startsim) = 1;
d.shk_lgdp_gap.shk_lgdp_gap(startsim) = 1;
d.shk_ls.shk_ls(startsim) = -1;
d.shk_rn.shk_rn(startsim) = 1;
d.shk_prem.shk_prem(startsim) = 1;

%% Simulates the shocks using the command 'simulate'. For this model
% 'm' and respective database 'd.{shock_name} are used. Results are written 
% in object 's.{shock_name}'. Command 'dboverlay' merges the historical 
% database d.{shock_name} with simulation database s.{shock_name}. 
for i=1:length(listshocks);    
    s.(listshocks{i}) = simulate(m,d.(listshocks{i}),startsim:endsim,'deviation',true);
    s.(listshocks{i}) = dboverlay(d.(listshocks{i}),s.(listshocks{i}));

% % Cumulative output gap
% s.(listshocks{i}).cum_gap = tseries(startsim-1,0);
%     for ii = 1:length(startsim:endsim)
%     s.(listshocks{i}).cum_gap(startsim:endsim) = s.(listshocks{i}).cum_gap{-1} + s.(listshocks{i}).lgdp_gap;
%     end
% s.(listshocks{i}).cum_gap.comment = (listshocks{i});
% disp('Cumulative output gap for');
% s.(listshocks{i}).cum_gap 

end

%% Graphs
% Makes graphs for all the shocks
Plotrng = startsim : endsim;

Heading = 'Model Properties';

x = report('heading',Heading,'dateformat','YYYY:P','color', true);

for i = 1:length(listshocks);

x = newpage(x);

x = opengraph(x);
  Heading = listtitles{i};
  x = tag(x,Heading,'fontsize','large');

  subplot(2,3,1); hold on;
  plot(Plotrng,[s.(listshocks{i}).dot4_cpi, s.(listshocks{i}).dot_cpi],'linewidth',2,{'color'},{'r';'b'}); title('Inflation'); 
  legend('y-o-y','q-o-q','location','best');
  grid on; box off;
  
  subplot(2,3,2); hold on;
  plot(Plotrng,[s.(listshocks{i}).rn],'linewidth',2); title('Interest Rate'); 
  grid on; box off;
  
  subplot(2,3,3); hold on;
  plot(Plotrng,[s.(listshocks{i}).dot_s],'linewidth',2); title('Nominal Exchange Rate (q-o-q)'); 
  grid on; box off;
  
  subplot(2,3,4); hold on;
  plot(Plotrng,[s.(listshocks{i}).lgdp_gap s.(listshocks{i}).cum_gap],'linewidth',2,{'color'},{'b';'r'},{'linestyle'},{'-';'-.'}); title('Output Gap'); legend('Output Gap','Cumulative Gap','location','best'); 
  grid on; box off;
  
  subplot(2,3,5); hold on;
  plot(Plotrng,s.(listshocks{i}).rr_gap,'linewidth',2); title('Real Interest Rate Gap'); 
  grid on; box off;
  
  subplot(2,3,6); hold on;
  plot(Plotrng,s.(listshocks{i}).lz_gap,'linewidth',2); title('Real Exchange Rate Gap'); 
  grid on; box off;
  
x = getgraph(x);

end

%% Disinflationary Shock, 1%
% Creates a steady-state database (a database filled with steady-state
% values) and sets the target to be lower for 1% over the simulation
% horizon
d.disinfl = sstatedb(m,startsim-4:endsim+40);
d.disinfl.target(startsim+1:endsim+40) =  d.disinfl.target - 1;

% Sets a simulation plan where the target is taken from the database (lower
% for 1% in comparison to the initial value)
simplan = plan(m,startsim+1:endsim+40);
simplan = exogenize(simplan,'target',startsim+1:endsim+40);
simplan = endogenize(simplan,'shk_target',startsim+1:endsim+40);

% Simulates the disinflation
s.disinfl = simulate(m,d.disinfl,startsim:endsim+40,'plan',simplan);

% Graph
% Makes graph for the disinflation
x = newpage(x);

x = opengraph(x);
  Heading = '1% Disinflation';
  x = tag(x,Heading,'fontsize','large');

  subplot(2,3,1); hold on;
  plot(Plotrng,[s.disinfl.dot4_cpi, s.disinfl.dot_cpi],'linewidth',2); title('Inflation'); 
  legend('y-o-y','q-o-q','location','best');
  grid on; box off;
  
  subplot(2,3,2); hold on;
  plot(Plotrng,[s.disinfl.rn],'linewidth',2); title('Interest Rate'); 
  grid on; box off;
  
  subplot(2,3,3); hold on;
  plot(Plotrng,[s.disinfl.dot_s],'linewidth',2); title('Nominal Exchange Rate (q-o-q)'); 
  grid on; box off;
  
  subplot(2,3,4); hold on;
  plot(Plotrng,[s.disinfl.lgdp_gap s.disinfl.cum_gap],'linewidth',2,{'color'},{'b';'r'},{'linestyle'},{'-';'-.'}); title('Output Gap'); legend('Output Gap','Cumulative Gap','location','best'); 
  grid on; box off;
  
  subplot(2,3,5); hold on;
  plot(Plotrng,s.disinfl.rr_gap,'linewidth',2); title('Real Interest Rate Gap'); 
  grid on; box off;
  
  subplot(2,3,6); hold on;
  plot(Plotrng,s.disinfl.lz_gap,'linewidth',2); title('Real Exchange Rate Gap'); 
  grid on; box off;
  
x = getgraph(x);

%% Saves all graphs
%% The default is set to generate a PostScript output, but it can be
%% changed to a .pdf (Acrobat) file by:
compile(x,'Report_Shocks.pdf');
compile(x,'Report_Shocks.ps','display',false);